from fastapi import APIRouter, HTTPException
from typing import Dict, Optional

from reviewsense.api.schemas.UserMessage import UserMessage
from reviewsense.api.schemas.product_review_input import ProductReviewInput
from reviewsense.services.BotService import BotService
from reviewsense.services.ReviewService import ReviewService


router = APIRouter()
review_service = ReviewService()

@router.post("/calculate_ratings/", response_model=Dict[str, Optional[float]])
async def calculate_ratings(input_data: ProductReviewInput):
    """
    Endpoint to calculate feature-specific ratings
    
    Args:
        input_data (ProductReviewInput): Input data for review processing
    
    Returns:
        Dictionary of feature ratings
    """
    # Add new review if provided
    if input_data.new_review:
        review_service.add_review(
            product_id=input_data.product_id, 
            review=input_data.new_review
        )
    
    # Fetch and analyze reviews
    reviews_by_feature = review_service.fetch_reviews(
        product_id=input_data.product_id, 
        features=input_data.features
    )
    
    # Generate and return feature ratings
    return review_service.generate_feature_ratings(reviews_by_feature)

@router.post("/chat")
async def chat(user_message: UserMessage):
    """
    Endpoint to handle user messages and return chatbot responses.
    """
    if not user_message.message:
        raise HTTPException(status_code=400, detail="Message cannot be empty")
    bot_service = BotService()

    # Handle a query
    response = bot_service.handle_user_query(user_message.message)

    # Or use individual components
    return {"response": response}
