# src/reviewsense/core/llm.py
from langchain_core.messages import BaseMessage
from langchain_core.prompts import ChatPromptTemplate
from langchain_groq import ChatGroq
from functools import lru_cache

from sqlalchemy.testing.plugin.plugin_base import logging


@lru_cache()
def get_llm():
    """Get singleton LLM instance"""
    return ChatGroq(
        model_name="llama-3.3-70b-versatile",
        temperature=1,
        groq_api_key="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"
    )




def generate_response(
        query: str,
        context: str,
        max_context_length: int = 2000
) -> str:
    """
    Generate response using LLM for ecommerce product queries.

    Args:
        query: User query string
        context: Context string containing product details and reviews
        max_context_length: Maximum allowed length for context

    Returns:
        Generated response string

    Raises:
        ValueError: If input parameters are invalid
        Exception: For other processing errors
    """
    # Input validation
    if not query or not query.strip():
        raise ValueError("Query cannot be empty")
    if not context or not context.strip():
        raise ValueError("Context cannot be empty")

    try:
        # Preprocess context
        context = context.strip()
        if len(context) > max_context_length:
            context = context[:max_context_length] + "..."

        llm_template = """
            Your task is to act as an ecommerce product expert assistant.
            Analyze the provided product information and reviews carefully.

            Guidelines:
            - Provide accurate product-specific information based on the context
            - Keep responses concise and focused on the query
            - Include relevant product features and customer feedback
            - If information is not in the context, acknowledge the limitation
            - Maintain a helpful and professional tone

            CONTEXT:
            {context}

            QUESTION: {input}

            YOUR ANSWER:
            """

        # Create and execute the chain
        chain = ChatPromptTemplate.from_template(llm_template) | get_llm()

        result = chain.invoke({
            "context": context,
            "input": query
        })

        # Extract response text based on message type
        if isinstance(result, BaseMessage):
            response = result.content
        elif isinstance(result, str):
            response = result
        else:
            response = str(result)

        # Validate response
        if not response or not response.strip():
            raise ValueError("Empty response received from LLM")

        return response.strip()

    except Exception as e:
        return "I apologize, but I encountered an error while processing your request. Please try again or contact support if the issue persists."
