from dataclasses import dataclass
from typing import List, Dict, Optional, Tuple, Any
from langchain_astradb import AstraDBVectorStore

from logging import getLogger
from functools import lru_cache

from langchain_core.output_parsers import JsonOutputParser
from langchain_core.prompts import ChatPromptTemplate

from reviewsense.core.llm import get_llm, generate_response
from reviewsense.services.retrieval import get_vector_store

logger = getLogger(__name__)


@dataclass
class ProductContext:
    product_name: str
    product_reviews: str


class BotServiceError(Exception):
    """Base exception for bot service errors"""
    pass


class VectorStoreError(BotServiceError):
    """Exception for vector store related errors"""
    pass


class ReviewProcessor:
    @staticmethod
    def clean_review(review: str) -> str:
        """Clean and format a single review"""
        return review.strip() + ('.' if not review.strip().endswith('.') else '')

    @staticmethod
    def process_reviews(doc_content: str) -> List[str]:
        """Process document content into cleaned review sentences"""
        return [
            ReviewProcessor.clean_review(sentence)
            for sentence in doc_content.split('.')
            if sentence.strip()
        ]


class BotService:
    def __init__(self, vector_store: Optional[AstraDBVectorStore] = None):
        self.vector_store = vector_store or self._initialize_vector_store()
        self.llm = get_llm()

    @staticmethod
    @lru_cache(maxsize=1)
    def _initialize_vector_store() -> AstraDBVectorStore:
        """Initialize and cache vector store connection"""
        try:
            return get_vector_store()
        except Exception as e:
            logger.error(f"Failed to initialize vector store: {e}")
            raise VectorStoreError(f"Vector store initialization failed: {e}")

    def get_product_reviews(
            self,
            product_id: str,
            query: str,
            k: int = 1
    ) -> str:
        """
        Retrieve and combine product reviews from the vector store.

        Args:
            product_id: Product identifier
            query: Search query
            k: Number of results to retrieve

        Returns:
            Combined review string
        """
        try:
            results = self.vector_store.similarity_search_with_score_id(
                query=query,
                k=k,
                filter={"product_name": product_id}
            )

            all_reviews = []
            for doc, _, _ in results:
                reviews = ReviewProcessor.process_reviews(doc.page_content)
                all_reviews.extend(reviews)

            return ' '.join(all_reviews)

        except Exception as e:
            logger.error(f"Error retrieving product reviews: {e}")
            raise VectorStoreError(f"Failed to retrieve reviews: {e}")

    def retrieve_context(
            self,
            query: str,
            search_query: str,
            k: int = 1
    ) -> Optional[ProductContext]:
        """
        Retrieve context from vector store.

        Args:
            query: User query
            search_query: Search query for vector store
            k: Number of results to retrieve

        Returns:
            ProductContext if found, None otherwise
        """
        try:
            as_retriever = self.vector_store.as_retriever(search_type="similarity_score_threshold",
                                                          search_kwargs={"k": 1, "score_threshold": 0.8}, )
            results = as_retriever.invoke(search_query)

            if not results:
                logger.info("No results found for query")
                return None

            document = results[0]
            product_name = document.metadata.get('product_name')

            if not product_name:
                logger.warning("Product name missing in document metadata")
                return None

            return ProductContext(
                product_name=product_name,
                product_reviews=self.get_product_reviews(query=query, product_id=product_name)
            )

        except Exception as e:
            logger.error(f"Error retrieving context: {e}")
            raise VectorStoreError(f"Context retrieval failed: {e}")

    def handle_user_query(self, query: str) -> str:
        """
        Process user query end-to-end.

        Args:
            query: User query

        Returns:
            Generated response
        """
        try:
            context = self.retrieve_context(query=query, search_query=query)

            if not context:
                return "I'm sorry, I couldn't find relevant information."

            formatted_context = f"""Product Information:
            Name: {context.product_name}

            Customer Reviews:
            {context.product_reviews}"""
            return generate_response(
                query=query,
                context=formatted_context
            )

        except BotServiceError as e:
            logger.error(f"Error handling user query: {e}")
            return "I apologize, but I encountered an error processing your request."
