from pydantic import BaseModel
from langchain.prompts import ChatPromptTemplate
from langchain_core.output_parsers import JsonOutputParser
from typing import List, Optional
from reviewsense.core.llm import get_llm

class FeatureReviews(BaseModel):
    feature_reviews: List[dict]  # Each review will include a sentence and sentiment

class FeatureExtractor:
    def __init__(self):
        self.llm = get_llm()
        self.parser = self._create_reviews_parser()
        self.prompt = self._create_extraction_prompt()
    
    def _create_reviews_parser(self) -> JsonOutputParser:
        """Create JSON parser for feature-specific reviews extraction"""
        return JsonOutputParser()
    
    def _create_extraction_prompt(self) -> ChatPromptTemplate:
        """Create prompt for extracting feature-specific reviews with enhanced rules and sentiment analysis."""
        template = """Extract sentences about the given feature from the list of reviews.

        Rules:
        - Extract only parts discussing the specific feature.
        - Remove unrelated parts connected by 'and' or 'but'.
        - Keep original wording and capitalization.
        - If there is only one review, apply the same rules to extract sentences about the feature.

        Reviews: {reviews}
        Feature: {feature}

        Return only the parts discussing the specific feature and perform sentiment analysis for each extracted sentence in this JSON format:
        {{
            "feature_reviews": [
                {{
                    "sentence": "relevant sentence 1",
                    "sentiment": "positive/negative/neutral"
                }},
                {{
                    "sentence": "relevant sentence 2",
                    "sentiment": "positive/negative/neutral"
                }}
            ]
        }}
        """
        return ChatPromptTemplate.from_template(template)

    def extract_feature_reviews(self, reviews: List[str], feature: str) -> List[dict]:
        """
        Extract feature-specific sentences from reviews with sentiment analysis
        
        Args:
            reviews: List of review texts
            feature: Target feature to extract
        
        Returns:
            List[dict]: Feature-specific sentences with sentiment analysis
        """
        try:
            chain = self.prompt | self.llm | self.parser
            reviews_text = "\n".join(reviews)
            
            result = chain.invoke({
                "reviews": reviews_text,
                "feature": feature
            })
            
            parsed_data = FeatureReviews(**result)  # Validate and parse the result
            return parsed_data.feature_reviews
            
        except Exception as e:
            print(f"Error extracting feature reviews: {e}")
            return []
